/*

*/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

const int MAX_INPUT = 1023;		// maximum letters of input
const int MAX_VAR = 30;       // maximum number of variables
const int MAX_VAR_LEN = 32;	// maximum length of variables
const int TRUE = 1;
const int FALSE = 0;
const int MAX_EXPR_LEN = 1023;
const int STACK_SIZE = 1024;

void error(const char*);	//global error handler

enum token_types {  VARIABLE, EOS, AND, OR, NOT, LP, RP };

#include "as4.h"    //class defs and member funct defs

#define DEBUG(x) do { if (debug_mode == 1) printf x; } while(0)

int debug_mode;


int main(int argc, char argv[])
{
    char str[MAX_EXPR_LEN+1];
    char kb[MAX_EXPR_LEN+1];
    int num_lines;
    parser parse;
    postfix pf;

    if (argc > 1) 
	{
	    printf("There was some argument. So enabling debugging mode.\n\n");
	    debug_mode = 1;
	}
    else
	debug_mode = 0;
	    
    FILE* fp = fopen("problem.txt", "r");
    FILE* fpout = fopen("solution.txt", "w");

    if (fp == NULL)
	{
	    printf("Cannot open problem.txt \n");
	    return 1;
	}

    fgets(kb, MAX_EXPR_LEN, fp);
    if (kb[strlen(kb)-1] == 13) kb[strlen(kb)-1] = 0;
    if (kb[strlen(kb)-1] == 10) kb[strlen(kb)-1] = 0;

    fscanf(fp, "%d\n", &num_lines);

    while(num_lines>0)
	{
	    char statement[MAX_EXPR_LEN+1];

	    fgets(statement, MAX_EXPR_LEN, fp);
	    if (statement[strlen(statement)-1] == 13) statement[strlen(statement)-1] = 0;
	    if (statement[strlen(statement)-1] == 10) statement[strlen(statement)-1] = 0;

	    sprintf(str, "~ ( %s ) || (%s)", kb, statement);

	    DEBUG(("\nExpression = %s\n", str));

	    parse.set_input(str);

	    parse.convert_postfix();

	    int tot_vars = parse.get_num_vars();

	    DEBUG(("Postfix: %s\n", parse.get_postfix()));
	    DEBUG(("Total number of variables: %d\n", tot_vars));

	    identifier_table* tabptr = parse.get_table();

	    int final_eval = 1;

	    for (int j=0; j<tot_vars; j++)
		DEBUG( ("%s\t", tabptr->get_var_at(j)) );

	    DEBUG(("eval\n"));

	    for (int i = 0; i < (1 << tot_vars); i++)
		{
		    //DEBUG(("\nbeginning next iteration: \n"));
		    for (int j=0; j<tot_vars; j++)
			{
			    tabptr->set_value(j, ((i & (1 << j)) == 0? 0:1) );
			    DEBUG( ("%d\t", ((i & (1 << j)) == 0? 0:1)));
			}
		    int eval = pf.evaluate(parse.get_postfix(), tabptr);

		    DEBUG(("eval=%d\n", eval));

		    if (eval != 1) final_eval = 0;
		}

	    parse.destroy();

	    DEBUG(("Final Eval: "));
	    if (final_eval == 1)
		{
		    DEBUG(("VALID\n"));
		    fprintf(fpout, "VALID\n");
		}
	    else
		{
		    DEBUG(("NOT VALID\n"));
		    fprintf(fpout, "NOT VALID\n");
		}

	    num_lines--;
	}

    fclose(fp);
    fclose(fpout);
    return 0;
}

void error(const char* str)
{
    fprintf(stderr, "ERROR: %s\n", str);
    exit(1);
}
